#include "bldc.h"

/* This isn't as efficient as an interrupt-driven source; there's an
   extra "stop" bit in the transmission stream because we can't set
   the TX data register soon enough.  However, we can't set the baud
   rate for 115200 accurately enough, and the extra stop bit keeps the
   host from getting out of sync.  */

#define CONSOLE SCI2

#define RXBUFSZ 16
static char rxbuf[RXBUFSZ];
static volatile int rxput, rxget, rxcount;

#define TXBUFSZ 256
static char txbuf[TXBUFSZ];
static volatile int txput, txget, txcount;

static volatile int tx_idle = 1;

static long hz = 10000000;

static void __attribute__((interrupt))
sci2_rx_handler (void)
{
  int b;
  b = CONSOLE.RDR;
  rxbuf[rxput++] = b;
  rxput %= RXBUFSZ;
  rxcount ++;
}

static void __attribute__((interrupt))
sci2_tx_handler (void)
{
  int b;
  if (txcount > 0)
    {
      b = txbuf[txget++];
      CONSOLE.TDR = b;
      txget %= TXBUFSZ;
      txcount --;
      tx_idle = 0;
    } else {
      tx_idle = 1;
  }
}

static void
tty2_init_port (int baud)
{
  int i;

    /* Disable, internal clock, div-1 */
  CONSOLE.SCR.BYTE = 0x00;

  /* async, 8N1, PCLK (40MHz) */
  CONSOLE.SMR.BYTE = 0x00;

  /* Disable smart card mode */
  CONSOLE.SCMR.BYTE = 0x00;

  /* div-8 mode in BRR */
  CONSOLE.SEMR.BYTE = 0x10;

  /* Calculate baud rate. */
#if 1
  /* 1,250,000 baud - use 1,263,157 for FT232R, 1.05% off */
  CONSOLE.BRR = 1;
#else
  CONSOLE.BRR = (hz*4) / (16 * baud) - 1;
#endif

  for (i=2200; i; --i)
    asm("nop");

  /* Enable transmitter, receiver, tx/rx interrupts */
  CONSOLE.SCR.BYTE = 0xf0;
}

void
tty2_set_hz (long new_hz)
{
  hz = new_hz;
}



void
tty2_init ()
{
  MSTP(SCI2) = 0;

  /* Enable SCI2 input buffer - PB.[56] */
  PORTB.DDR.BIT.B6 = 0;
  PORTB.ICR.BYTE = 0xff;
  PORTB.DDR.BIT.B5 = 1;
  IOPORT.PFFSCI.BIT.SCI2S = 0;

  PORTB.DDR.BIT.B4 = 1;
  PORTB.DDR.BIT.B5 = 1;
  PORTB.DDR.BIT.B6 = 1;
  PORTB.DDR.BIT.B7 = 1;

  rxput = rxget = rxcount = 0;

  /* SCI2 interrupts */
  _vectors[VECT(SCI2,RXI2)] = sci2_rx_handler;
  IEN(SCI2,RXI2) = 1;
  IPR(SCI2,RXI2) = 5;
  _vectors[VECT(SCI2,TXI2)] = sci2_tx_handler;
  IEN(SCI2,TXI2) = 1;
  IPR(SCI2,TXI2) = 5;

  tty2_init_port (115200);
}

void
tty2_putc (char c)
{
  int i;
  U2RTS = 1;
#if 0
  /* TEND = 0:transmitting, 1:done */
  while (! (CONSOLE.SSR.BIT.TEND))
    ;
  CONSOLE.TDR = c;
#endif
  while (txcount == TXBUFSZ)
    asm("");
  asm("clrpsw i");
  if (tx_idle)
    {
      tx_idle = 0;
      CONSOLE.TDR = c;
    }
  else
    {
      txbuf[txput++] = c;
      txput %= TXBUFSZ;
      txcount ++;
    }
  asm("setpsw i");
  U2RTS = 0;
}

void
tty2_puts (char *s)
{
  while (*s)
    tty2_putc (*s++);
}

char
tty2_getc ()
{
  char rv;
  while (rxcount == 0)
    ;
  rv = rxbuf[rxget++];
  rxget %= RXBUFSZ;
  rxcount --;
  return rv;
}

int
tty2_ready ()
{
  return rxcount;
}

int
tty2_idle ()
{
  return !txcount;
}

int
tty2_busy ()
{
  return txcount;
}

int
tty2_can_accept (int chars)
{
  return txcount < TXBUFSZ - chars - 1;
}

void
tty2_put_marker ()
{
  tty2_putc(0xcc | (motor_pwm_ticks & 3));
}

void
tty2_putc_encoded (int c)
{
  c &= 0xff;
  if ((c & 0xf8) == 0xc8)
    {
      tty2_putc(0xc8);
      tty2_putc(c - 0x80);
    }
  else
    tty2_putc (c);
}

void
tty2_putw_encoded (int i)
{
  tty2_putc_encoded (i & 0xff);
  tty2_putc_encoded (i >> 8);
}
